/*
 * phoenix_CodePress - Real Time Syntax Highlighting Editor written in JavaScript - http://phoenix_CodePress.org/
 *
 * Copyright (C) 2007 Fernando M.A.d.S. <fermads@gmail.com>
 *
 * Developers:
 *		Fernando M.A.d.S. <fermads@gmail.com>
 *		Michael Hurni <michael.hurni@gmail.com>
 * Contributors:
 *		Martin D. Kirk
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU Lesser General Public License as published by the Free Software Foundation.
 *
 * Read the full licence: http://www.opensource.org/licenses/lgpl-license.php
 */

phoenix_CodePress = {
	scrolling : false,

	initialize : function() {
		prefs = phoenix_getPrefs ();

		if (typeof(editor) == 'undefined' && !arguments[0]) // && prefs.getBoolPref('syntaxHighlighting') == false
			return;

		body = document.getElementsByTagName('body')[0];
		body.innerHTML = body.innerHTML.replace(/\n/g,"");

		/*
		   8: backspace
		  13: enter
		  32: space
		  46: delete
		  59; ;
		  62: .
		 186: ;
		*/

		chars = '|32|46|62|8|'; // charcodes that trigger syntax highlighting
		cc = '\u2009'; // carret char
		editor = document.getElementsByTagName('body')[0];
		document.designMode = 'on';
		document.addEventListener('keypress', this.keyHandler, true);
		window.addEventListener('scroll', function() { if(!phoenix_CodePress.scrolling) phoenix_CodePress.syntaxHighlight('scroll') }, false);
		completeChars = this.getCompleteChars();
		completeEndingChars =  this.getCompleteEndingChars();
	},

	// added by me (phoenix)
	exit : function() {
		document.removeEventListener('keypress', this.keyHandler, true);
	},

	// treat key bindings
	keyHandler : function(evt) {
    	keyCode = evt.keyCode;
		charCode = evt.charCode;
		fromChar = String.fromCharCode(charCode);

		if ((evt.ctrlKey || evt.metaKey) && evt.shiftKey && charCode != 90) // shortcuts = ctrl||appleKey+shift+key!=z(undo)
		{
			phoenix_CodePress.shortcuts(charCode?charCode:keyCode);
		}

		else if ((completeEndingChars.indexOf('|' + fromChar + '|') != -1 || completeChars.indexOf('|' + fromChar + '|') != -1) && prefs.getBoolPref('autoComplete'))
		{
			if (!phoenix_CodePress.completeEnding(fromChar))
				phoenix_CodePress.complete(fromChar);
		}

		else if (chars.indexOf('|' + charCode + '|') != -1) // syntax highlighting
		{
			top.setTimeout(function() { phoenix_CodePress.syntaxHighlight('generic'); }, 100);
		}

		else if (evt.keyCode == 13) // enter key is pressed
		{
			evt.preventDefault();
			phoenix_CodePress.insertCode("\n", false);
			top.setTimeout(function() { phoenix_CodePress.syntaxHighlight('generic'); }, 100);
		}

		else if (charCode == 118 && evt.ctrlKey) // handle paste
		{
		 	top.setTimeout(function() { phoenix_CodePress.syntaxHighlight('generic'); }, 100);
		}

		else if (charCode == 99 && evt.ctrlKey) // handle cut
		{
		 	//alert(window.getSelection().getRangeAt(0).toString().replace(/\t/g,'FFF'));
		}

	},

	// put cursor back to its original position after every parsing
	findString : function() {
		if (self.find(cc))
			window.getSelection().getRangeAt(0).deleteContents();
	},

	// split big files, highlighting parts of it
	split : function (code, flag) {
		if (flag == 'scroll')
		{
			this.scrolling = true;
			return (code);
		}

		else
		{
			this.scrolling = false;
			mid = code.indexOf(cc);
			if (mid - 100000 < 0)
			{
				ini = 0;
				end = 200000;
			}

			else if (mid + 100000 > code.length)
			{
				ini = code.length - 200000;
				end = code.length;
			}

			else
			{
				ini = mid - 100000;
				end = mid + 100000;
			}

			code = code.substring(ini, end);
			return (code);
		}
	},

	getEditor : function() {
		body = document.getElementsByTagName('body')[0];
		if (!body.innerHTML)
			return (body);

		return (document.getElementsByTagName('body')[0]);
	},

	// syntax highlighting parser
	syntaxHighlight : function(flag) {

		if (flag != 'init')
			window.getSelection().getRangeAt(0).insertNode(document.createTextNode(cc));

		editor = phoenix_CodePress.getEditor();
		o = editor.innerHTML;
		o = o.replace(/<br>/mig, '\n');
		o = o.replace(/<\/li><\/ol>/mig, '');
		o = o.replace(/<\/li>/mig, '\n');
		o = o.replace(/<[^>]*>/mig, '');
		x = z = this.split(o, flag);
		x = x.replace(/\n/g, '<br>');

		if (prefs.getBoolPref('syntaxHighlighting'))
		{
			if (arguments[1] && arguments[2])
				x = x.replace(arguments[1], arguments[2]);

			for (i = 0; i < phoenix_Language.syntax.length; i++)
				x = x.replace(phoenix_Language.syntax[i].input,phoenix_Language.syntax[i].output);
		}

		if (prefs.getBoolPref('lineNumbers'))
		{
			x = "<a>" + x + "</a>";
			x = x.replace(/<br>/mig, "<li>");
			x = "<ol><li>" + x + "</ol>";
		}

		editor.innerHTML = x;
		editor.innerHTML = (flag == 'scroll') ? x : o.split(z).join(x);
		if (flag != 'init')
			this.findString();
	},

	getLastWord : function() {
		var rangeAndCaret = phoenix_CodePress.getRangeAndCaret();
		words = rangeAndCaret[0].substring(rangeAndCaret[1] - 40, rangeAndCaret[1]);
		words = words.replace(/[\s\n\r\);\W]/g, '\n').split('\n');

		return (words[words.length-1].replace(/[\W]/gi,'').toLowerCase());
	},

	complete : function(trigger) {
		window.getSelection().getRangeAt(0).deleteContents();
		var complete = phoenix_Language.complete;
		for (var i = 0; i < complete.length; i++)
		{
			if (complete[i].input == trigger)
			{
				var pattern = new RegExp('\\' + trigger + cc);
				var content = complete[i].output.replace(/\$0/g, cc);
				parent.setTimeout(function () { phoenix_CodePress.syntaxHighlight('complete', pattern, content)}, 0); // wait for char to appear on screen
			}
		}
	},

	getCompleteChars : function() {
		var cChars = '';
		for (var i = 0; i < phoenix_Language.complete.length; i++)
			cChars += '|' + phoenix_Language.complete[i].input;

		return (cChars + '|');
	},

	getCompleteEndingChars : function() {
		var cChars = '';
		for (var i = 0;i < phoenix_Language.complete.length; i++)
			cChars += '|' + phoenix_Language.complete[i].output.charAt(phoenix_Language.complete[i].output.length - 1);

		return (cChars + '|');
	},

	completeEnding : function(trigger) {
		var range = window.getSelection().getRangeAt(0);
		try {
			range.setEnd(range.endContainer, range.endOffset+1)
		}
		catch(e) {
			return (false);
		}

		var next_character = range.toString()
		range.setEnd(range.endContainer, range.endOffset - 1)
		if (next_character != trigger)
		{
			return (false);
		}
		else
		{
			range.setEnd(range.endContainer, range.endOffset + 1)
			range.deleteContents();
			return (true);
		}
	},

	shortcuts : function() {
		var cCode = arguments[0];
		if (cCode == 13)
			cCode = '[enter]';

		else if (cCode==32)
			cCode = '[space]';

		else
			cCode = '[' + String.fromCharCode(charCode).toLowerCase() + ']';

		for (var i = 0; i < phoenix_Language.shortcuts.length; i++)
		{
			if (phoenix_Language.shortcuts[i].input == cCode)
				this.insertCode(phoenix_Language.shortcuts[i].output, false);
		}
	},

	// https://developer.mozilla.org/En/DOM/Range.endOffset
	getRangeAndCaret : function() {
		var range = window.getSelection().getRangeAt(0);
		var range2 = range.cloneRange();
		var node = range.endContainer;
		var caret = range.endOffset;
		range2.selectNode(node);

		return ([range2.toString(), caret]);
	},

	insertCode : function(code,replaceCursorBefore) {
		var range = window.getSelection().getRangeAt(0);
		var node = window.document.createTextNode(code);
		var selct = window.getSelection();
		var range2 = range.cloneRange();

		// Insert text at cursor position
		selct.removeAllRanges();
		range.deleteContents();
		range.insertNode(node);

		// Move the cursor to the end of text
		range2.selectNode(node);
		range2.collapse(replaceCursorBefore);
		selct.removeAllRanges();
		selct.addRange(range2);
	},

	// get code from editor
	getCode : function() {
		if (!document.getElementsByTagName('body')[0] || editor.innerHTML == '')
			editor = phoenix_CodePress.getEditor();

		var code = editor.innerHTML;
		code = code.replace(/<br>/g,'\n');
		code = code.replace(/\u2009/g,'');
		code = code.replace(/<.*?>/g,'');
		code = code.replace(/&lt;/g,'<');
		code = code.replace(/&gt;/g,'>');
		code = code.replace(/&amp;/gi,'&');

		return (code);
	},

	// put code inside editor
	setCode : function() {
		var code = arguments[0];
		code = code.replace(/\u2009/gi,'');
		code = code.replace(/&/gi,'&amp;');
		code = code.replace(/</g,'&lt;');
		code = code.replace(/>/g,'&gt;');
		editor.innerHTML = code;

		if (code == '')
			document.getElementsByTagName('body')[0].innerHTML = '';
	},
}


phoenix_Language={};
window.addEventListener('load', function() { phoenix_CodePress.initialize('new'); }, true);